/*
* createTorusVAO
* Based on freeGLUT glut*Torus code by John Fay
* Modified by Alex Clarke, Feb. 2012
* 
* Given:
*    innerRadius: inner radius of the torus
*    outerRadius: outer radius of the torus
*    nSides: number of segments in cross section of torus
*    nRings: number of segments around the torus
*    vertexPositionIndex: location in shader for vertex position data
*    vertexNormalIndex: location in shader for vertex normal data
*
* Returns:
*    vao: a vertex array object loaded with GLfloat data for 
*         a torus as specified by input parameters. 
*         Render with GL_TRIANGLES
*         User is responsible for deleting this VAO when it is 
*         no longer needed.
*         See: glDeleteVertexArrays
*    verts: number of vertices in the vao
*/
#include <cmath>
#include <cassert>
#include "Angel.h"

void createTorusVAO( GLfloat innerRadius, GLfloat outerRadius, GLint nSides, GLint nRings, 
	GLuint vertexPositionIndex, GLuint vertexNormalIndex, 
	GLuint &vao, GLuint &verts)
{
	GLfloat  iradius = innerRadius, oradius = outerRadius, phi, psi, dpsi, dphi;
	GLfloat *vertex, *normal, *vTris, *nTris;
	int    i, j;
	GLfloat spsi, cpsi, sphi, cphi ;

	if ( nSides < 1 ) nSides = 1;
	if ( nRings < 1 ) nRings = 1;

	/* Allocate the data arrays */
	int nVerts = 4 * nSides * nRings;
	int triVerts =  2 * 3 * 4 * (nSides-1) * (nRings-1);
	vertex = new GLfloat[nVerts];
	normal = new GLfloat[nVerts];
	vTris = new GLfloat[triVerts];
	nTris = new GLfloat[triVerts];

	dpsi =  GLfloat(2.0 * M_PI / (double)(nRings - 1)) ;
	dphi = GLfloat(-2.0 * M_PI / (double)(nSides - 1));
	psi  = 0.0f;

	for( j=0; j<nRings; j++ )
	{
		cpsi = cos ( psi ) ;
		spsi = sin ( psi ) ;
		phi = 0.0;

		for( i=0; i<nSides; i++ )
		{
			int offset = 4 * ( j * nSides + i ) ;
			cphi = cos ( phi ) ;
			sphi = sin ( phi ) ;
			vertex[offset + 0] = cpsi * ( oradius + cphi * iradius ) ;
			vertex[offset + 1] = spsi * ( oradius + cphi * iradius ) ;
			vertex[offset + 2] = sphi * iradius  ;
			vertex[offset + 3] = 1 ;
			normal[offset + 0] = cpsi * cphi ;
			normal[offset + 1] = spsi * cphi ;
			normal[offset + 2] = sphi ;
			normal[offset + 3] = 0 ;
			phi += dphi;
		}

		psi += dpsi;
	}

	int v = 0;
	//Process Quads for faces
	for( i=0; i<nSides-1; i++ )
	{
		for( j=0; j<nRings-1; j++ )
		{
			int offset = 4 * ( j * nSides + i ) ;
			int k;
			//Break Faces into two triangles
			//Triangle 1
			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[offset + k];
				vTris[v++] = vertex[offset + k];
			}
			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[offset + 4 + k];
				vTris[v++] = vertex[offset + 4 + k];
			}

			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[offset + 4 * nSides + 4 + k];
				vTris[v++] = vertex[ offset + 4 * nSides + 4 + k];
			}

			//Triangle 2
			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[ offset + 4 * nSides + k];
				vTris[v++] = vertex[ offset + 4 * nSides + k];
			}

			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[offset+k];
				vTris[v++] = vertex[offset+k];
			}
			for (k = 0; k < 4; k++)
			{
				nTris[v] = normal[offset + 4 * nSides + 4 + k];
				vTris[v++] = vertex[ offset + 4 * nSides + 4 + k];
			}

		}
	}

	//Load data into a vertex buffer
	glGenVertexArrays( 1, &vao );
	glBindVertexArray( vao );

	GLuint buffer;
	glGenBuffers( 1, &buffer );
	glBindBuffer( GL_ARRAY_BUFFER, buffer );
	glBufferData( GL_ARRAY_BUFFER, 2*sizeof(GLfloat)*triVerts, NULL, GL_STATIC_DRAW );
	glBufferSubData( GL_ARRAY_BUFFER, 0,        sizeof(GLfloat)*triVerts, vTris );
	glBufferSubData( GL_ARRAY_BUFFER, sizeof(GLfloat)*triVerts, sizeof(GLfloat)*triVerts, nTris );
	glEnableVertexAttribArray(vertexPositionIndex);
	glVertexAttribPointer( vertexPositionIndex, 4, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(0) );
	glEnableVertexAttribArray(vertexNormalIndex);
	glVertexAttribPointer( vertexNormalIndex,   4, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET(sizeof(GLfloat)*triVerts) );


	free ( vertex );
	free ( normal );
	free ( nTris );
	free ( vTris );
	verts = triVerts/4;
}
